CONFIG_ITEMS = [
    "access",
    "banner",
    "branches",
    "branches-global",
    "cli",
    "cords",
    "cords-global",
    "data-trending",
    "email",
    "ftp",
    "groups",
    "ldap",
    "lines",
    "lines-global",
    "network",
    "outlets",
    "outlets-global",
    "ocps",
    "phases",
    "phases-global",
    "ports",
    "radius",
    "sensors/adc",
    "sensors/adc-global",
    "sensors/contact",
    "sensors/fan",
    "sensors/fan-global",
    "sensors/humidity",
    "sensors/humidity-global",
    "sensors/temp",
    "sensors/temp-global",
    "sensors/water",
    "snmp",
    "sntp",
    "syslog",
    "system",
    "tacacs",
    "units",
    "users/ldap",
    "users/local",
    "users/snmpv3",
    "users/tacacs",
    "web",
    "wlan",
    "ztp",
]

LOCAL_USER_LEVELS = {
    "view only": 10,
    "on only": 11,
    "reboot only": 12,
    "user": 13,
    "power user": 14,
    "admin": 15,
}

SUPPORTED_OUTLET_ACTIONS = ["off", "on", "reboot"]

SUPPORTED_RESTART_ACTIONS = [
    "factory",
    "factory keep network",
    "new firmware",
    "new ssh keys",
    "new x509 certificate",
    "normal",
]
